@@ -44,26 +44,34 @@ split_decimal <- function(x, sigfig, digits = NULL, sci_mod = NULL, si = FALSE,
   "!!!!!!DEBUG `v(mnt)`"
 
   if (!is.null(sci_mod)) {
-    exp <- fix_exp(num, compute_exp(mnt, sigfig, digits), fixed_exponent, sci_mod, si)
+    exp <- compute_exp_display(mnt, sigfig, digits)
+    exp <- fix_exp(num, exp, fixed_exponent, sci_mod, si)
     "!!!!!!DEBUG `v(exp)`"
+    unit <- attr(exp, "unit")
 
     # Must divide by 10^exp, because 10^-exp may not be representable
     # for very large values of exp
     mnt_idx <- which(num & mnt != 0)
     mnt[mnt_idx] <- safe_divide_10_to(mnt[mnt_idx], exp[mnt_idx])
     "!!!!!!DEBUG `v(mnt)`"
-
-    exp_display <- exp
   } else {
     exp <- 0
     "!!!!!!DEBUG `v(exp)`"
+    unit <- NULL
+  }
+
+  if (is.null(sci_mod) || !is.null(fixed_exponent)) {
     exp_display <- rep_along(x, NA_integer_)
+  } else {
+    exp_display <- exp
   }
 
   if (is.null(digits)) {
     "!!!!!!DEBUG `v(sigfig)`"
     min_sigfig <- compute_min_sigfig(mnt)
+    "!!!!!!DEBUG `v(min_sigfig)`"
     round_mnt <- safe_signif(mnt, pmax(sigfig, min_sigfig, na.rm = TRUE))
+    "!!!!!!DEBUG `v(round_mnt)`"
     rhs_digits <- compute_rhs_digits(mnt, sigfig)
   } else if (digits >= 0) {
     "!!!!!!DEBUG `v(digits)`"
@@ -98,12 +106,15 @@ split_decimal <- function(x, sigfig, digits = NULL, sci_mod = NULL, si = FALSE,
     sigfig = sigfig,
     num = num,
     neg = neg,
-    lhs = format(lhs, scientific = FALSE, trim = TRUE),
+    # integer64 doesn't support format(trim = FALSE)
+    # trimws() is unnecessarily slow
+    lhs = sub("^ +", "", format(lhs, scientific = FALSE)),
     lhs_zero = (lhs == 0),
     rhs = rhs,
     rhs_digits = rhs_digits,
     dec = dec,
     exp = exp_display,
+    unit = unit,
     si = si
   )
 
@@ -123,19 +134,19 @@ fix_exp <- function(num, exp, fixed_exponent, sci_mod, si) {
       exp <- max(exp, na.rm = TRUE)
     }
     "!!!!!!DEBUG `v(exp)`"
-    exp <- rep_along(num, as.integer(round(exp)))
+    exp <- as.integer(round(exp))
+    exp <- structure(rep_along(num, exp), unit = exp %|% 0L)
     exp[!num] <- NA_integer_
     "!!!!!!DEBUG `v(exp)`"
   }
 
   if (sci_mod != 1) {
-    exp <- as.integer(round(floor(exp / sci_mod) * sci_mod))
+    exp[] <- as.integer(round(floor(exp / sci_mod) * sci_mod))
     "!!!!!!DEBUG `v(exp)`"
   }
   if (si) {
     # Truncate very small and very large exponents
-    exp <- pmax(exp, -24L)
-    exp <- pmin(exp, 24L)
+    exp[] <- pmin(pmax(exp, -24L), 24L)
     "!!!!!!DEBUG `v(exp)`"
   }
 
@@ -167,7 +178,7 @@ safe_signif <- function(x, digits) {
 safe_divide_10_to <- function(x, y) {
   # Computes x / 10^y in a robust way
 
-  x / (10^y)
+  10^(log10(x) - y)
 }
 
 eps_2 <- 2 * .Machine$double.eps
@@ -186,7 +197,7 @@ within_tolerance <- function(x, y) {
   # Work around integer64 problem
   equal[x == y] <- FALSE
   "!!!!!!DEBUG `v(abs((x[equal] - y[equal]) * 2 ^ -l2x[equal]))`"
-  out[equal] <- abs((x[equal] - y[equal]) * 2 ^ -l2x[equal]) <= eps_2
+  out[equal] <- abs((x[equal] - y[equal]) * 2^-l2x[equal]) <= eps_2
   out
 }
 
@@ -202,6 +213,7 @@ compute_rhs_digits <- function(x, sigfig, offset = rep_along(x, 0)) {
 
   if (!is.integer(x) && !all(x == trunc(x), na.rm = TRUE)) {
     has_rhs <- (exp <= sigfig)
+    "!!!!!!DEBUG `v(has_rhs)`"
     rhs_digits[has_rhs] <- sigfig - 1 - exp[has_rhs]
 
     to_check <- rhs_digits > 0
@@ -215,6 +227,7 @@ compute_rhs_digits <- function(x, sigfig, offset = rep_along(x, 0)) {
       "!!!!!!DEBUG `v(val - round(val))"
 
       resid_zero <- within_tolerance(val, round(val))
+      "!!!!!!DEBUG `v(resid_zero)`"
       resid_zero[is.na(resid_zero)] <- FALSE
 
       rhs_digits[which_to_check][resid_zero] <-
@@ -223,6 +236,9 @@ compute_rhs_digits <- function(x, sigfig, offset = rep_along(x, 0)) {
       to_check[which_to_check][!resid_zero] <- FALSE
       to_check[rhs_digits == 0] <- FALSE
     }
+
+    "!!!!!!DEBUG `v(to_check)"
+    "!!!!!!DEBUG `v(rhs_digits)"
   }
 
   "!!!!!!DEBUG `v(rhs_digits)"
@@ -251,7 +267,8 @@ compute_extra_sigfig <- function(x) {
 
 LOG_10 <- log(10)
 
-compute_exp <- function(x, sigfig, digits) {
+compute_exp_display <- function(x, sigfig, digits = NULL) {
+  "!!!!!!DEBUG compute_exp_display(`v(x)`, `v(sigfig)`, `v(digits)`)"
   if (is.null(sigfig)) {
     sigfig <- abs(digits)
   }
@@ -264,10 +281,21 @@ compute_exp <- function(x, sigfig, digits) {
   # Division before log is the same as subtraction after log.
   # Using log1p for numerical stability.
   offset <- log1p(-5 * 10^(-sigfig - 1)) / LOG_10
+  "!!!!!!DEBUG `v(offset)`"
 
   ret <- rep_along(x, NA_integer_)
   nonzero <- which(x != 0 & is.finite(x))
   ret[nonzero] <- as.integer(floor(log10(x[nonzero]) - offset))
+  "!!!!!!DEBUG `v(ret)`"
+  ret
+}
+
+compute_exp <- function(x, sigfig) {
+  "!!!!!!DEBUG compute_exp(`v(x)`, `v(sigfig)`)"
+  ret <- rep_along(x, NA_integer_)
+  nonzero <- which(x != 0 & is.finite(x))
+  ret[nonzero] <- as.integer(floor(log10(x[nonzero])))
+  "!!!!!!DEBUG `v(ret)`"
   ret
 }
 
@@ -342,13 +370,18 @@ format_rhs <- function(s) {
   neg <- s$neg
   dec <- s$dec
   lhs_zero <- s$lhs_zero
-  rhs_num <- s$rhs_num
   rhs_digits <- s$rhs_digits
 
   # Digits on RHS of .
-  rhs_num <- sprintf("%.0f", abs(round(s$rhs * 10^(s$rhs_digits))))
+
+  # s$rhs * 10^(rhs_digits) should be almost an integer
+  rhs_val <- s$rhs * 10^rhs_digits
+
+  rhs_num <- sprintf("%.0f", abs(round(rhs_val)))
   rhs_num[rhs_num == "0"] <- ""
 
+  # Special treatment for leading zeros for some formatting,
+  # hand-rolling
   n_zeros <- pmax(0, rhs_digits - get_extent(rhs_num))
   rhs_zero <- strrep("0", n_zeros)
 
@@ -366,7 +399,7 @@ format_rhs <- function(s) {
 
   rhs_col <- ifelse(dec,
     paste0(
-      style_num(rhs_underlined_zero, neg, !lhs_zero),
+      style_num(rhs_underlined_zero, neg, significant = !lhs_zero),
       style_num(rhs_underlined_num, neg)
     ),
     ""
@@ -404,7 +437,7 @@ style_num <- function(x, negative, significant = rep_along(x, TRUE)) {
 
 assemble_decimal <- function(x) {
   mantissa <- format_mantissa(x)
-  exp <- format_exp(x)
+  exp <- format_exp(x$exp, x$si)
 
   paste0(mantissa, exp)
 }
