@@ -56,6 +56,7 @@ test_that("glimpse calls tbl_sum() (#550)", {
 })
 
 test_that("output test for glimpse()", {
+  skip_if_not_installed("tibble")
   local_unknown_rows()
 
   expect_snapshot({
@@ -100,3 +101,59 @@ test_that("output test for glimpse()", {
     glimpse(nested_mtcars_tbl, width = 70L)
   })
 })
+
+test_that("format_glimpse_1() handles commas correctly", {
+  local_colors()
+
+  expect_snapshot({
+    "# Vector with commas"
+    format_glimpse_1(1:3)
+    format_glimpse_1(c("a", "b", "c"))
+    format_glimpse_1(c(1, NA, 3))
+
+    "# Empty and single-element vectors"
+    format_glimpse_1(integer())
+    format_glimpse_1(1)
+
+    "# Mixed content with commas"
+    format_glimpse_1(list(1:3, "a, b", NA))
+
+    # format_glimpse_1 with .inner = TRUE
+    "# Simple list elements"
+    format_glimpse_1(1:3, .inner = TRUE)
+    format_glimpse_1(letters[1:3], .inner = TRUE)
+
+    "# Nested lists"
+    format_glimpse_1(list(1:3), .inner = TRUE)
+    format_glimpse_1(list(list(1:3), list(4:6)), .inner = TRUE)
+
+    "# Mixed content"
+    format_glimpse_1(list(1:3, "a,b,c"), .inner = TRUE)
+    format_glimpse_1(list(list(1, 2, 3), letters[1:3]), .inner = TRUE)
+
+    "# Empty and NULL"
+    format_glimpse_1(list(), .inner = TRUE)
+    format_glimpse_1(NULL, .inner = TRUE)
+  })
+})
+
+test_that("color test for missing values", {
+  local_colors()
+
+  expect_snapshot({
+    "individual data types"
+    format_glimpse(df_all$a)
+    format_glimpse(df_all$b)
+    format_glimpse(df_all$c)
+    format_glimpse(df_all$d)
+    format_glimpse(df_all$e)
+    format_glimpse(df_all$f)
+    format_glimpse(df_all$g)
+    format_glimpse(df_all$h)
+    format_glimpse(df_all$i)
+
+    "tables"
+    glimpse(df_all)
+    glimpse(as.data.frame(df_all))
+  })
+})
